import re
import os
import socket
import asyncio
import math
import errno
from struct import unpack
from functools import reduce
import sys
import time
import datetime


from basic_logtools.filelog import LogFile

from networktools.time import timestamp
from networktools.library import my_random_string
'''
###############################################################################
CLASE ERYO  __authors__ = " David Pineda"
###############################################################################
Conexion TCP/IP con un determinado receptor.
'''
from .eryo_settings import (ERYO_FIELD_NAMES, ERYO_REC, ERYO_TABLE_NAMES,
                            BUFF_MAX_SIZE, TABLES_SIZES, ERYO_LIST, SYNC_MARKER_VALUE, BLOCKS_FLAGS,
                            ETN_reverse, TABLES_BY_FLAG, CONS, MSG_ID, signal_check)

from networktools.colorprint import rprint, gprint, bprint
"""
Check verification

int eryo_trailer_decode(char *buf, int start, eryo_trailer_t *ptr)
{
  int index=start;
  DECODE_BITS(16, buf, index, ptr->checksum);
  assert(index - start==sizeof(eryo_trailer_t));
  return index;
}

inidice = partida
decodificar 2 bytes (16 bits) a variable checksu,
compara,  la diferencia (index-start) con tamaño de eryo_trailer_y


"""


def convert_data(key, table_name, data):
    size_table = ERYO_REC[table_name]
    field_names = ERYO_FIELD_NAMES[key]
    bytes_table = TABLES_SIZES[table_name]
    try:
        unpacked = unpack(size_table, data)
    except Exception as e:
        print("No unpack on converta data", e)
        raise e
    table_dict = dict(zip(field_names, unpacked))
    return table_dict


def checksum(data):
    check_data = list(data)
    suma = sum(check_data[:-2])
    return suma


class Eryo:
    """ Class to connect to tcp port and parse ERYO messages """
    tipo = "ERYO"

    def __init__(self, **kwargs):
        station = kwargs.get('code')
        sock = kwargs.get('sock')
        self.timeout = kwargs.get('timeout', 1)
        self.host = kwargs.get('host')
        self.port = kwargs.get('port')
        self.sock = self.create_socket(None)
        self.loop = kwargs.get('loop')
        self.station = station
        self.sock.settimeout(self.timeout)
        self.msg_dict = {}
        self.msg_0 = None
        self.msg_bytes = None
        self.checksum = None
        self.rec_dict = {}
        self.batt_status = 0
        self.try_connect = 0
        self.id_columns = {}
        self.keys = []
        self.tables = ERYO_TABLE_NAMES
        self.status = False
        # log file for this station
        # create log instance
        log_path = kwargs.get('log_path', './logs')
        log_level = kwargs.get('log_level', 'INFO')
        self.logger = LogFile(self.class_name,
                              self.station,
                              self.host,
                              path=log_path,
                              base_level=log_level)
        # manage asyncronous clients
        self.logger.info("Log para %s" % self.station)
        self.idc = []
        self.clients = {}

    @property
    def class_name(self):
        return self.__class__.__name__

    def create_socket(self, sock):
        sock = None
        if sock is None:
            sock = socket.socket(
                socket.AF_INET, socket.SOCK_STREAM)
        else:
            sock = sock
        return sock

    def off_blocking(self):
        self.sock.setblocking(False)

    def on_blocking(self):
        self.sock.setblocking(True)

    async def connect(self):
        host = self.host
        port = self.port
        loop = self.loop
        address = (host, port)
        while not self.status:
            try:
                (reader, writer) = await asyncio.open_connection(
                    loop=loop,
                    host=host,
                    port=port)
                idc = await self.set_reader_writer(reader, writer)
                if self.hard_beat(idc):
                    self.status = True
                    self.logger.info("Conexion a %s realizada" % self.station)
                else:
                    info = writer.get_extra_info('peername')
                    self.logger.error(info)
            except socket.timeout as timeout:
                self.on_blocking()
                self.logger.exception(
                    "Error en conexión con %s:%s, error: %s" % (host, port, timeout))
                #print("Error de socket a ERYO en conexión %s" % timeout)
                self.status = False
                await asyncio.sleep(10)
                # await self.connect()
                # raise timeout
            except socket.error as e:
                self.status = False
                self.on_blocking()
                if e.errno == errno.ECONNREFUSED:
                    # print("Conexión rechazada en %s" %
                    #      self.station, file=sys.stderr)
                    self.logger.exception(e)
                    await asyncio.sleep(10)
                else:
                    # print("No se puede establecer conexión, de %s" %
                    #      self.station, file=sys.stderr)
                    self.logger.exception(e)
                    await asyncio.sleep(10)
        return idc

    # callback for create server:

    def hard_beat(self, idc):
        if idc in self.clients:
            reader = self.clients[idc]['reader']
            writer = self.clients[idc]['writer']
            if writer.can_write_eof():
                return True
            else:
                self.logger.error("no hard_beat")
                self.close(idc)
                self.status = False
                return False
        else:
            self.logger.error("no hard_beat, idc %s not client" % idc)
            self.close(idc)
            self.status = False
            return False

    def info(self, idc):
        if self.clients:
            writer = self.clients[idc]['writer']
            return writer.get_extra_info('peername')
        else:
            return None

    def set_idc(self):
        """
        Defines a new id for relation process-collect_task, check if exists
        """
        uin = 4
        idc = my_random_string(uin)
        while True:
            if idc not in self.idc:
                self.idc.append(idc)
                break
            else:
                idc = my_random_string(uin)
        return idc

    async def set_reader_writer(self, reader, writer):
        idc = self.set_idc()
        # self.log_info_client(writer)
        self.clients.update(
            {idc: {
                'reader': reader,
                'writer': writer
            }
            }
        )
        return idc

    def list_clients(self):
        for i in range(len(self.conss)):
            print(str(self.addrs[i]) + ":" + str(self.conns[i]))

    def close(self, idc):
        self.logger.error("La conexión se cerró en cliente %s" % idc)
        self.status = False
        reader = self.clients[idc]['reader']
        writer = self.clients[idc]['writer']
        writer.close()

    def stop(self):
        for idc in self.clients:
            self.close(idc)

    async def get_records(self):
        """
        Obtener bytes y traducir a un msg ERYO

        """
        idc = self.select_idc
        reader = self.clients[idc]['reader']
        writer = self.clients[idc]['writer']
        sync_marker_first = SYNC_MARKER_VALUE.get('first')
        sync_marker_second = SYNC_MARKER_VALUE.get('second')
        t0 = time.time()
        try:
            if writer.transport.is_closing():
                self.logger.error("La conexión se cerró %s" % writer)
                self.status = False
                raise Exception("Conexión perdida")
            try:
                done = False
                counter = 1
                data = b''
                byte_a, byte_b = (0, 0)
                suma_bytes = 0
                msg = {}
                position = 0
                while not done:
                    byte_a = await reader.readexactly(1)
                    while byte_a != sync_marker_first:
                        byte_a = await reader.readexactly(1)
                    byte_b = await reader.readexactly(1)
                    if byte_b == sync_marker_second:
                        # read next 3 bytes
                        resto_header = await reader.readexactly(3)
                        suma_bytes += 5
                        header = byte_a+byte_b+resto_header
                        data += header
                        data_dict = convert_data(1, "ERYO_HEADER", header)
                        data_dict['MESSAGE_ID'] = MSG_ID.get(
                            data_dict.get("MESSAGE_ID"))
                        msg["ERYO_HEADER"] = data_dict
                        # POSITION_BLOCK
                        byte_count = data_dict.get("BYTE_COUNT")
                        all_bytes = await reader.readexactly(byte_count-5)
                        data += all_bytes
                        key_2 = "POSITION_BLOCK"
                        PB_bytes = TABLES_SIZES.get(key_2)
                        suma_bytes += PB_bytes
                        PB_data = all_bytes[position:position+PB_bytes]
                        position += PB_bytes
                        PB_dict = convert_data(2, key_2, PB_data)
                        SITE_ID = PB_dict["SITE_ID"].decode("utf8")
                        PB_dict["SITE_ID"] = re.sub("\x00+$", "", SITE_ID)
                        SOLUTION_ID = PB_dict["SOLUTION_ID"].decode("utf8")
                        PB_dict["SOLUTION_ID"] = re.sub(
                            "\x00+$", "", SOLUTION_ID)
                        msg["POSITION_BLOCK"] = PB_dict
                        FLAGS = PB_dict.get('FLAGS')
                        bloques = list(filter(
                            lambda e: e[1] > 0,
                            [(block, FLAGS & value) for block, value in
                             BLOCKS_FLAGS.items()]))
                        for block_name, value in bloques:
                            if block_name == 'SAT_INFO':
                                HDR = "SAT_HDR"
                                INFO = "SAT_INFO_BLOCK"
                                nbytes = TABLES_SIZES.get(HDR)
                                suma_bytes += nbytes
                                data_block = all_bytes[position:position+nbytes]
                                position += nbytes
                                n = ETN_reverse.get(HDR)
                                data_dict = convert_data(n, HDR, data_block)
                                msg["SAT_INFO"] = {HDR: data_dict}
                                msg["SAT_INFO"]["DATA"] = []
                                NRO_SATS = data_dict.get('SAT_BLOCK_COUNT')
                                key = TABLES_BY_FLAG.get("SAT_INFO")[1]
                                for i in range(NRO_SATS):
                                    nbytes = TABLES_SIZES.get(INFO)
                                    suma_bytes += nbytes
                                    data_block = all_bytes[position:position+nbytes]
                                    position += nbytes
                                    n = ETN_reverse.get(INFO)
                                    data_dict = convert_data(
                                        n, INFO, data_block)
                                    key = "CONSTELLATION"
                                    data_dict[key] = CONS.get(
                                        data_dict.get(key))
                                    key = "SIGNAL_FLAGS"
                                    data_dict["SIGNALS"] = signal_check(
                                        data_dict.get(key))
                                    msg["SAT_INFO"]["DATA"].append(data_dict)
                            else:
                                for key in TABLES_BY_FLAG.get(block_name):
                                    nbytes = TABLES_SIZES.get(key)
                                    suma_bytes += nbytes
                                    data_block = all_bytes[position:position+nbytes]
                                    position += nbytes
                                    n = ETN_reverse.get(key)
                                    data_dict = convert_data(
                                        n, key, data_block)
                                    msg[key] = data_dict
                        # CHECKSUM
                        key = "ERYO_TRAILER"
                        nbytes = TABLES_SIZES.get(key)
                        suma_bytes += nbytes
                        checksum_bytes = all_bytes[position:position+nbytes]
                        n = ETN_reverse.get(key)
                        # checksum_bytes = await reader.readexactly(nbytes)
                        data_dict = convert_data(n, key, checksum_bytes)
                        msg[key] = data_dict
                        checksum_msg = data_dict.get("CHECKSUM")
                        chsum = checksum(data)
                        done = checksum_msg == chsum
                return done, msg
            except socket.timeout as timeout:
                self.logger.warning(
                    "La conexión se cerró, socket timeout %s" % timeout)
                self.on_blocking()
                self.logger.exception()
                self.status = False
                await asyncio.sleep(.1)
                raise timeout
            except Exception as ex:
                self.logger.error(
                    "La conexión se cerró, error en comunicación %s" % ex)
                self.close(idc)
                self.on_blocking()
                info = "Error %s en estación %s. Encabezado con data %s" % (
                    ex, self.station, data)
                self.logger.info(info)
                self.logger.error(ex)
                self.status = False
                await asyncio.sleep(.1)
                if ex == socket.timeout:
                    self.sock = self.create_socket(None)
                    self.on_blocking()
                    await self.connect()
                raise ex
        except socket.timeout as timeout:
            self.logger.warning(
                "La conexión se cerró, socket timeout %s" % timeout)
            self.on_blocking()
            self.logger.exception()
            self.status = False
            await asyncio.sleep(.1)
            raise timeout
        except Exception as ex:
            self.logger.error(
                "La conexión se cerró, error en comunicación %s" % ex)
            self.close(idc)
            self.on_blocking()
            info = "Error %s en estación %s. Encabezado con data %s" % (
                ex, self.station, data)
            print(info)
            self.logger.info(info)
            self.logger.error(ex)
            self.status = False
            await asyncio.sleep(.1)
            if ex == socket.timeout:
                self.sock = self.create_socket(None)
                self.on_blocking()
                await self.connect()
            raise ex

    async def get_message_header(self, idc):
        self.select_idc = idc



from dataprotocols.gsof import Gsof
import asyncio
import functools
import os
import signal
import click
import json
import time

# TEST
CODE = 'HSCO'
IP = "hsco.dyndns.org"
PORT = 5018


async def run_test(loop, code, host, port, limit=2000):
    ginput = dict(host=host,
                  port=port,
                  code=code,
                  timeout=5,
                  sock=None,
                  loop=loop)
    gsof_test = "eryo_test.json"
    limit_counter = limit
    counter = 0    
    print("Conexion a->", ginput)
    gsof = Gsof(**ginput)
    print("GSOF object", gsof)
    idc = await gsof.connect()
    print("IDC->", idc)
    with open(gsof_test, "a+") as gsof_file:
        gsof_file.write("[")
        while counter <= limit_counter:
            try:
                print("Hard beat")
                print(gsof.hard_beat(idc))
                print("MSG recv>")
                await gsof.get_message_header(idc)
                print("Header msg->", gsof.msg_bytes)
                done, msg = await gsof.get_records()
                if done:
                    print("Eryo", msg)
                    json.dump(msg, gsof_file, indent=2)
                    gsof_file.write(",\n")
            except Exception as ex:
                gsof_file.write("]")
                print("Error %s" % ex)
                loop.close()
            except KeyboardInterrupt as ke:
                gsof_file.write("]")
                loop.close()
                print(ke)
            counter += 1
        gsof_file.write("]")



@click.command()
@click.option("--code", default="HSCO", help="Código de estación")
@click.option("--host", default="hsco.dyndns.org", help="URL de estación")
@click.option("--port", default=5018, help="Nro de puerto")
@click.option("--limit", default=2000, help="Nro de mensajes")
def run_gsof(code, host, port, limit):
    loop = asyncio.get_event_loop()

    try:
        loop.run_until_complete(run_test(loop, code, host, port, limit))
    except Exception as ex:
        print("Exception %s" %ex)
        loop.call_soon_threadsafe(loop.stop)
        print(ex)
    except KeyboardInterrupt as ke:
        print("Exception %s" %ex)        
        loop.call_soon_threadsafe(loop.stop)
        print(ke)
    finally:
        loop.close()
    

if __name__=='__main__':
    run_gsof()



from dataprotocols.eryo import Eryo
import asyncio
import functools
import os
import signal
import json
import time
import click
# TEST
#  CHDA,200.9.100.194,12323
# CODE = 'CHDA'
# IP = "200.9.100.194"
# PORT = 12323


async def run_test(loop, code, host, port, limit=2000):
    ginput = dict(host=host,
                  port=port,
                  code=code,
                  timeout=5,
                  sock=None,
                  loop=loop)
    eryo = Eryo(**ginput)
    idc = await eryo.connect()
    eryo_test = "eryo_test.json"
    limit_counter = limit
    counter = 0
    with open(eryo_test, "a+") as eryo_file:
        eryo_file.write("[")
        while counter <= limit_counter:
            try:
                eryo.hard_beat(idc)
                await eryo.get_message_header(idc)
                done, msg = await eryo.get_records()
                if done:
                    print("Eryo", msg)
                    json.dump(msg, eryo_file, indent=2)
                    eryo_file.write(",\n")
            except Exception as ex:
                eryo_file.write("]")
                print("Error %s" % ex)
                loop.close()
            except KeyboardInterrupt as ke:
                eryo_file.write("]")
                loop.close()
                print(ke)
            counter += 1
        eryo_file.write("]")

# CODE = 'CHDA'
# IP = "200.9.100.194"
# PORT = 12323

@click.command()
@click.option("--code", default="CHDA", help="Código de estación")
@click.option("--host", default="200.9.100.194", help="URL de estación")
@click.option("--port", default=12323, help="Nro de puerto")
@click.option("--limit", default=2000, help="Nro de mensajes")
def run_eryo(code, host, port, limit):
    loop = asyncio.get_event_loop()

    try:
        loop.run_until_complete(run_test(loop, code, host, port, limit))
    except Exception as ex:
        print("Exception %s" %ex)
        loop.call_soon_threadsafe(loop.stop)
        print(ex)
    except KeyboardInterrupt as ke:
        print("Exception %s" %ex)        
        loop.call_soon_threadsafe(loop.stop)
        print(ke)
    finally:
        loop.close()
    
if __name__=='__main__':
    run_eryo()



from struct import unpack
import socket
import asyncio
import math
import errno
from functools import reduce

import sys
import time
import datetime

from pathlib import Path
import os
from basic_logtools.filelog import LogFile

from networktools.time import timestamp, now
from networktools.library import my_random_string

from networktools.colorprint import rprint, bprint

'''
###############################################################################
CLASE GSOF  __authors__ = "Henry T. Berglund ----> David Pineda"
###############################################################################
Conexion TCP/IP con un determinado receptor.
Updated to python3.5 by David Pineda @pineiden dpineda@csn.uchile.cl

'''

try:
    from .gsof_settings import gsof_field_names, gsof_rec, gsof_table_names, logfile
except Exception:
    from gsof_settings import gsof_field_names, gsof_rec, gsof_table_names, logfile

def checksum256(data_check):
    """Calculate checksum"""
    # ord: string -> unicode int
    # map(a,b): opera a en b
    # list entrega lista o arrays
     #formula: 	(Status + type + length + data bytes) modulo 256
    data_bytes = data_check.get("DATA_BYTES")
    checksum = data_check.get("CHECKSUM")
    chk256  = sum(data_bytes) % 256
    return  checksum == chk256

class Gsof:
    """ Class to connect to tcp port and parse GSOF messages """
    tipo = "GSOF"

    def __init__(self, **kwargs):
        station = kwargs.get('code')
        sock = kwargs.get('sock')
        self.timeout = kwargs.get('timeout', 1)
        self.host = kwargs.get('host')
        self.port = kwargs.get('port')
        self.sock = self.create_socket(None)
        self.loop = kwargs.get('loop')
        self.station = station
        self.sock.settimeout(self.timeout)
        self.msg_dict = {}
        self.msg_0 = None
        self.msg_bytes = None
        self.checksum = None
        self.rec_dict = {}
        self.batt_status = 0
        self.try_connect = 0
        self.id_columns = {}
        self.keys = []
        self.tables = gsof_table_names
        self.status = False

        # manage asyncronous clients
        self.idc = []
        self.clients = {}
        log_path = kwargs.get('log_path', './logs')
        log_level = kwargs.get('log_level', 'INFO')
        self.logger = LogFile(self.class_name,
                              station,
                              self.host,
                              path=log_path,
                              base_level=log_level)
        # manage asyncronous clients
        self.logger.info("Log para %s" % self.station)

    @property
    def class_name(self):
        return self.__class__.__name__

    def create_socket(self, sock):
        sock = None
        if sock is None:
            sock = socket.socket(
                socket.AF_INET, socket.SOCK_STREAM)
        else:
            sock = sock
        return sock

    def off_blocking(self):
        self.sock.setblocking(False)

    def on_blocking(self):
        self.sock.setblocking(True)

    async def connect(self):
        host = self.host
        port = self.port
        loop = self.loop
        address = (host, port)
        while not self.status:
            try:
                (reader, writer) = await asyncio.open_connection(
                    loop=loop,
                    host=host,
                    port=port)
                idc = await self.set_reader_writer(reader, writer)
                if self.hard_beat(idc):
                    self.status = True
                    self.logger.info("Conexion a %s realizada" % self.station)
                else:
                    info = writer.get_extra_info('peername')
                    self.logger.error(info)
            except socket.timeout as timeout:
                self.on_blocking()
                self.logger.error(
                    "Error en conexión con %s:%s, error: %s" % (host, port, timeout))
                #print("Error de socket a GSOF en conexión %s" % timeout)
                self.status = False
                await asyncio.sleep(10)
                # await self.connect()
                # raise timeout
            except socket.error as e:
                self.status = False
                self.on_blocking()
                if e.errno == errno.ECONNREFUSED:
                    # print("Conexión rechazada en %s" %
                    #      self.station, file=sys.stderr)
                    self.logger.exception("Error al conectar %s, station %s" %(e, self.station))
                    await asyncio.sleep(10)
                else:
                    # print("No se puede establecer conexión, de %s" %
                    #      self.station, file=sys.stderr)
                    self.logger.exception("Otro tipo de error al conectar %s, station %s" %(e, self.station))
                    await asyncio.sleep(10)

            await asyncio.sleep(.2)

        return idc

    # callback for create server:

    def hard_beat(self, idc):
        tnow = now()
        if idc in self.clients.keys():
            reader = self.clients[idc]['reader']
            writer = self.clients[idc]['writer']
            if writer.can_write_eof():
                return True
            else:
                print("Fail hard beat to %s, at %s" %(self.station, tnow))
                self.logger.error("no hard_beat, at %s" %tnow)
                self.close(idc)
                self.status = False
                return False
        else:
            self.logger.error("no hard_beat, idc %s not client, code station %s" % (idc, self.station))
            self.close(idc)
            self.status = False
            return False

    def info(self, idc):
        if self.clients:
            writer = self.clients[idc]['writer']
            return writer.get_extra_info('peername')
        else:
            return None

    def set_idc(self):
        """
        Defines a new id for relation process-collect_task, check if exists
        """
        uin = 4
        idc = my_random_string(uin)
        while True:
            if idc not in self.idc:
                self.idc.append(idc)
                break
            else:
                idc = my_random_string(uin)
        return idc

    async def set_reader_writer(self, reader, writer):
        idc = self.set_idc()
        # self.log_info_client(writer)
        self.clients.update(
            {idc: {
                'reader': reader,
                'writer': writer
            }
            }
        )
        return idc

    def list_clients(self):
        for i in range(len(self.conss)):
            print(str(self.addrs[i]) + ":" + str(self.conns[i]))

    def close(self, idc):
        self.logger.error("La conexión se cerró en cliente %s" % idc)
        self.status = False
        reader = self.clients[idc]['reader']
        writer = self.clients[idc]['writer']
        writer.close()

    def stop(self):
        for idc in self.clients:
            self.close(idc)

    async def get_message_header(self, idc):
        # receive data from source
        # max amount of data
        # N ammount of bites
        self.data_check = {}
        reader = self.clients[idc]['reader']
        writer = self.clients[idc]['writer']
        self.timestamp = timestamp()
        BytesData = 7
        bufsize = BytesData
        msg_field_names = ('STX', 'STATUS', 'TYPE', 'LENGTH',
                           'T_NUM', 'PAGE_INDEX', 'MAX_PAGE_INDEX')

        try:
            if writer.transport.is_closing():
                self.logger.error("La conexión se cerró %s" % writer)
                self.status = False
                raise Exception("Conexión perdida para idc %s station %s" %(idc, self.station))
            data = None

            try:
                data = await reader.readexactly(bufsize)
                self.msg_dict = dict(zip(
                    msg_field_names,
                    unpack('>' + str(bufsize) + 'B', data)))
                campos_unpack =unpack('>%dB'%bufsize, data)
                self.msg_dict = dict(zip(msg_field_names, campos_unpack))
                status = self.msg_dict['STATUS']
                bin_status = bin(status)
                batt_status = bin_status[3]
                self.msg_dict["LOW_BATTERY"] = bool(int(batt_status))
                self.msg_bytes = await reader.readexactly(self.msg_dict['LENGTH']-3)
                self.msg_0 = self.msg_bytes                
                check_etx =  await reader.readexactly(n=2)
                (checksum, etx) = unpack('>2B', check_etx)
                self.data_check["CHECKSUM"] = checksum                
                self.data_check["DATA_BYTES"] = list(self.msg_bytes+data[1:])
                self.checksum = checksum256(self.data_check)
                self.msg_dict["CHECKED"] = self.checksum
            except socket.timeout as timeout:
                self.logger.exception(
                    "La conexión se cerró, socket timeout %s, STATION %S" % (timeout, self.station))
                self.on_blocking()
                self.logger.exception("Esperando mucho tiempo")
                self.status = False
                raise timeout
            except Exception as ex:
                self.logger.exception(
                    "La conexión se cerró, error en comunicación %s" % ex)
                self.close(idc)
                self.on_blocking()
                info = "Error %s en estación %s. Encabezado con data %s" % (
                    ex, self.station, data)
                print(info)
                self.logger.info(info)
                self.logger.error(ex)
                self.status = False
                raise ex

        except asyncio.IncompleteReadError as e:
            self.logger.exception("Error de mensaje incompleto %s" % e)
            self.logger.error(
                "Error de mensaje esperado en n bytes %s" % e.expected)
            self.logger.error("Error de mensaje recibido %s" % e.partial)
            raise e

        except socket.timeout:
            self.close()
            self.sock = self.create_socket(None)
            self.on_blocking()
            raise socket.timeout

        except socket.error as ex:
            self.close()
            print("Error de conexión %s error %s" % (self.station, ex))
            self.logger.error(ex)
            self.sock = self.create_socket(None)
            self.on_blocking()
            raise ex

    async def get_records(self):
        msg = {}        
        while self.msg_bytes:
            # READ THE FIRST TWO BYTES FROM RECORD HEADER
            try:
                record_type, record_length = unpack('>2B', self.msg_bytes[0:2])
            except Exception as ex:
                self.status = False
                qex = "Error %s en estación %s, tabla %s" % (
                    ex, self.station, 'Header')
                self.logger.debug(qex)
                if ex == socket.timeout:
                    self.on_blocking()
                    loop = asyncio.get_event_loop()
                    loop.run_until_complete(self.connect())

            self.msg_bytes = self.msg_bytes[2:]
            try:
                msg_table = self.select_record(record_type, record_length)
                msg.update(msg_table)
            except Exception as ex:
                self.status = False
                self.logger.error(ex)
                print("No hay mensaje que traducir")
                self.logger.error("Desconectando debido a error en mensaje")
                self.logger.error(
                    "Reconectando debido a error en mensaje, luego de desconexión")
                self.on_blocking()
                return False, {}
        msg["HEADER"] = self.msg_dict
        done = self.checksum
        return done, msg

    def select_record(self, record_type, record_length):
        field_names = gsof_field_names
        rec_input = gsof_rec
        msg = []
        table_name = self.tables.get(record_type)
        timestamp = self.timestamp
        try:
            rec_field_names = field_names.get(record_type)
            if not record_type in [13, 14, 33, 48] and record_type in field_names.keys():
                # print("Base recibida")
                try:
                    rec_values = unpack(
                        rec_input[record_type], self.msg_bytes[:record_length])
                    msg = dict(list(zip(rec_field_names, rec_values)))
                    msg['TIMESTAMP'] = timestamp
                    self.rec_dict.update({table_name: msg})
                    self.msg_bytes = self.msg_bytes[record_length:]
                except Exception as ex:
                    self.status = False
                    self.logger.exception("Error al hacer unpack",ex)
                    raise ex

            elif record_type in [13, 14, 33, 48] and record_type in field_names.keys():
                L = len(self.msg_bytes)-1
                bNUM = self.msg_bytes[0:1]
                try:
                    code = rec_input[record_type][0]
                    NUM_OF_SVS = unpack(code, bNUM)  # char-> pass to int
                except Exception as ex:
                    self.status = False
                    qex = "Error %s en estación %s, tabla %s" % (
                        ex, self.station, record_type)
                    self.logger.exception(qex)
                    raise ex
                u = int(L/NUM_OF_SVS[0])
                # Acortar el NSVS quitando primer byte
                self.msg_bytes = self.msg_bytes[1:]
                # for field in range(len(rec_field_names)):
                #    self.rec_dict[rec_field_names[field]] = []
                for sat in range(NUM_OF_SVS[0]):
                    try:
                        code = rec_input[record_type][1]
                        rec_values = unpack(
                            code, self.msg_bytes[0:u])  # why [0:10]
                        msg_k = dict(list(zip(rec_field_names, rec_values)))
                        msg_k['TIMESTAMP'] = timestamp
                        msg.append(msg_k)
                    except Exception as ex:
                        self.status = False
                        self.logger.debug(ex)
                        raise ex
                    self.msg_bytes = self.msg_bytes[u:]
                    tname = table_name+"_"+str(sat)
                    self.rec_dict.update({table_name: msg})
            else:
                """Unknown record type? Skip it for now!"""
                # print record_type
                msg = self.msg_bytes[record_length + 2:]
                self.msg_bytes = self.msg_bytes[record_length + 2:]
        except Exception as ex:
            self.status = False
            raise ex
        return {table_name: msg}





from setuptools import setup, find_packages

setup(name='dataprotocols',
      version='0.3.0',
      description='DataProtocols is a set of classes that implement protocols for data acquisition',
      url='http://gitlab.csn.uchile.cl/dpineda/dataprotocols',
      author='David Pineda Osorio',
      author_email='dpineda@csn.uchile.cl',
      install_requires=['Click','networktools', 'basic-logtools'],
      scripts=[
          'dataprotocols/gsof/test_gsof.py',
          'dataprotocols/eryo/test_eryo.py'          
      ],      
      entry_points={
        'console_scripts':["gsof = dataprotocols.gsof.test_gsof:run_gsof",
                          "eryo = dataprotocols.eryo.test_eryo:run_eryo"]
        },
      packages=find_packages(),
      include_package_data=True,
      license='MIT',
      zip_safe=False)
